@@ -1,4 +1,4 @@
-// CT - simple-minded unit testing for C
+/* CT - simple-minded unit testing for C */
 
 #include <signal.h>
 #include <string.h>
@@ -14,18 +14,20 @@
 #include <errno.h>
 #include <sys/time.h>
 #include <stdint.h>
+#include <inttypes.h>
 #include "internal.h"
 #include "ct.h"
 
 
 static char *curdir;
 static int rjobfd = -1, wjobfd = -1;
+int fail = 0; /* bool */
 static int64 bstart, bdur;
-static int btiming; // bool
+static int btiming; /* bool */
 static int64 bbytes;
-static const int64 Second = 1000 * 1000 * 1000;
-static const int64 BenchTime = Second;
-static const int MaxN = 1000 * 1000 * 1000;
+enum { Second = 1000 * 1000 * 1000 };
+enum { BenchTime = Second };
+enum { MaxN = 1000 * 1000 * 1000 };
 
 
 
@@ -39,6 +41,7 @@ nstime()
 }
 
 #else
+#	include <time.h>
 
 static int64
 nstime()
@@ -51,7 +54,7 @@ nstime()
 #endif
 
 void
-ctlogpn(char *p, int n, char *fmt, ...)
+ctlogpn(const char *p, int n, const char *fmt, ...)
 {
     va_list arg;
 
@@ -66,8 +69,14 @@ ctlogpn(char *p, int n, char *fmt, ...)
 void
 ctfail(void)
 {
-    fflush(stdout);
-    fflush(stderr);
+    fail = 1;
+}
+
+
+void
+ctfailnow(void)
+{
+    fflush(NULL);
     abort();
 }
 
@@ -75,7 +84,6 @@ ctfail(void)
 char *
 ctdir(void)
 {
-    mkdir(curdir, 0700);
     return curdir;
 }
 
@@ -116,7 +124,7 @@ ctsetbytes(int n)
 
 
 static void
-die(int code, int err, char *msg)
+die(int code, int err, const char *msg)
 {
     putc('\n', stderr);
 
@@ -159,7 +167,7 @@ waittest(Test *ts)
     if (pid == -1) {
         die(3, errno, "wait");
     }
-    killpg(pid, 9);
+    killpg(pid, SIGKILL);
 
     for (t=ts; t->f; t++) {
         if (t->pid == pid) {
@@ -182,7 +190,10 @@ start(Test *t)
 {
     t->fd = tmpfd();
     strcpy(t->dir, TmpDirPat);
-    mktemp(t->dir);
+    if (mkdtemp(t->dir) == NULL) {
+	die(1, errno, "mkdtemp");
+    }
+    fflush(NULL);
     t->pid = fork();
     if (t->pid < 0) {
         die(1, errno, "fork");
@@ -199,7 +210,10 @@ start(Test *t)
         }
         curdir = t->dir;
         t->f();
-        _exit(0);
+        if (fail) {
+            ctfailnow();
+        }
+        exit(0);
     }
     setpgid(t->pid, t->pid);
 }
@@ -228,7 +242,7 @@ static void
 copyfd(FILE *out, int in)
 {
     ssize_t n;
-    char buf[1024]; // arbitrary size
+    char buf[1024]; /* arbitrary size */
 
     while ((n = read(in, buf, sizeof(buf))) != 0) {
         if (fwrite(buf, 1, n, out) != (size_t)n) {
@@ -238,15 +252,17 @@ copyfd(FILE *out, int in)
 }
 
 
-// Removes path and all of its children.
-// Writes errors to stderr and keeps going.
-// If path doesn't exist, rmtree returns silently.
+/*
+Removes path and all of its children.
+Writes errors to stderr and keeps going.
+If path doesn't exist, rmtree returns silently.
+*/
 static void
 rmtree(char *path)
 {
     int r = unlink(path);
     if (r == 0 || errno == ENOENT) {
-        return; // success
+        return; /* success */
     }
     int unlinkerr = errno;
 
@@ -285,7 +301,10 @@ runbenchn(Benchmark *b, int n)
     int outfd = tmpfd();
     int durfd = tmpfd();
     strcpy(b->dir, TmpDirPat);
-    mktemp(b->dir);
+    if (mkdtemp(b->dir) == NULL) {
+	die(1, errno, "mkdtemp");
+    }
+    fflush(NULL);
     int pid = fork();
     if (pid < 0) {
         die(1, errno, "fork");
@@ -304,9 +323,13 @@ runbenchn(Benchmark *b, int n)
         ctstarttimer();
         b->f(n);
         ctstoptimer();
-        write(durfd, &bdur, sizeof bdur);
-        write(durfd, &bbytes, sizeof bbytes);
-        _exit(0);
+        if (write(durfd, &bdur, sizeof bdur) != sizeof bdur) {
+            die(3, errno, "write");
+        }
+        if (write(durfd, &bbytes, sizeof bbytes) != sizeof bbytes) {
+            die(3, errno, "write");
+        }
+        exit(0);
     }
     setpgid(pid, pid);
 
@@ -314,7 +337,7 @@ runbenchn(Benchmark *b, int n)
     if (pid == -1) {
         die(3, errno, "wait");
     }
-    killpg(pid, 9);
+    killpg(pid, SIGKILL);
     rmtree(b->dir);
     if (b->status != 0) {
         putchar('\n');
@@ -337,17 +360,17 @@ runbenchn(Benchmark *b, int n)
 }
 
 
-// rounddown10 rounds a number down to the nearest power of 10.
+/* rounddown10 rounds a number down to the nearest power of 10. */
 static int
 rounddown10(int n)
 {
     int tens = 0;
-    // tens = floor(log_10(n))
+    /* tens = floor(log_10(n)) */
     while (n >= 10) {
         n = n / 10;
         tens++;
     }
-    // result = 10**tens
+    /* result = 10**tens */
     int i, result = 1;
     for (i = 0; i < tens; i++) {
         result *= 10;
@@ -356,18 +379,20 @@ rounddown10(int n)
 }
 
 
-// roundup rounds n up to a number of the form [1eX, 2eX, 5eX].
+/* roundup rounds n up to a number of the form [1eX, 2eX, 5eX]. */
 static int
 roundup(int n)
 {
     int base = rounddown10(n);
-    if (n == base)
-        return n;
+    if (n <= base)
+        return base;
     if (n <= 2*base)
-        return 2 * base;
+        return 2*base;
+    if (n <= 3*base)
+        return 3*base;
     if (n <= 5*base)
-        return 5 * base;
-    return 10 * base;
+        return 5*base;
+    return 10*base;
 }
 
 
@@ -400,23 +425,23 @@ runbench(Benchmark *b)
     runbenchn(b, n);
     while (b->status == 0 && b->dur < BenchTime && n < MaxN) {
         int last = n;
-        // Predict iterations/sec.
+        /* Predict iterations/sec. */
         int nsop = b->dur / n;
         if (nsop == 0) {
             n = MaxN;
         } else {
             n = BenchTime / nsop;
         }
-        // Run more iterations than we think we'll need for a second (1.5x).
-        // Don't grow too fast in case we had timing errors previously.
-        // Be sure to run at least one more than last time.
-        n = max(min(n+n/2, 100*last), last+1);
-        // Round up to something easy to read.
+        /* Run more iterations than we think we'll need for a second (1.2x).
+        Don't grow too fast in case we had timing errors previously.
+        Be sure to run at least one more than last time. */
+        n = max(min(n+n/5, 100*last), last+1);
+        /* Round up to something easy to read. */
         n = roundup(n);
         runbenchn(b, n);
     }
     if (b->status == 0) {
-        printf("%8d\t%10lld ns/op", n, b->dur/n);
+        printf("%8d\t%10" PRId64 " ns/op", n, b->dur/n);
         if (b->bytes > 0) {
             double mbs = 0;
             if (b->dur > 0) {
@@ -495,14 +520,17 @@ report(Test *t)
 }
 
 
-int
+static int
 readtokens()
 {
     int n = 1;
     char c, *s;
-    if ((s = strstr(getenv("MAKEFLAGS"), " --jobserver-fds="))) {
-        rjobfd = (int)strtol(s+17, &s, 10);  // skip " --jobserver-fds="
-        wjobfd = (int)strtol(s+1, NULL, 10); // skip comma
+    char *v = getenv("MAKEFLAGS");
+    if (v == NULL)
+        return n;
+    if ((s = strstr(v, " --jobserver-fds="))) {
+        rjobfd = (int)strtol(s+17, &s, 10);  /* skip " --jobserver-fds=" */
+        wjobfd = (int)strtol(s+1, NULL, 10); /* skip comma */
     }
     if (rjobfd >= 0) {
         fcntl(rjobfd, F_SETFL, fcntl(rjobfd, F_GETFL)|O_NONBLOCK);
@@ -514,14 +542,16 @@ readtokens()
 }
 
 
-void
+static void
 writetokens(int n)
 {
     char c = '+';
     if (wjobfd >= 0) {
         fcntl(wjobfd, F_SETFL, fcntl(wjobfd, F_GETFL)|O_NONBLOCK);
         for (; n>1; n--) {
-            write(wjobfd, &c, 1); // ignore error; nothing we can do anyway
+            if (write(wjobfd, &c, 1) != 1) {
+                /* ignore error; nothing we can do anyway */
+            }
         }
     }
 }
