@@ -1,14 +1,14 @@
+#include "dat.h"
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#include "dat.h"
 
 static uint64 next_id = 1;
 
 static int cur_prime = 0;
 
-static job all_jobs_init[12289] = {0};
-static job *all_jobs = all_jobs_init;
+static Job *all_jobs_init[12289] = {0};
+static Job **all_jobs = all_jobs_init;
 static size_t all_jobs_cap = 12289; /* == primes[0] */
 static size_t all_jobs_used = 0;
 
@@ -23,7 +23,7 @@ _get_job_hash_index(uint64 job_id)
 }
 
 static void
-store_job(job j)
+store_job(Job *j)
 {
     int index = 0;
 
@@ -40,7 +40,7 @@ store_job(job j)
 static void
 rehash(int is_upscaling)
 {
-    job *old = all_jobs;
+    Job **old = all_jobs;
     size_t old_cap = all_jobs_cap, old_used = all_jobs_used, i;
     int old_prime = cur_prime;
     int d = is_upscaling ? 1 : -1;
@@ -52,7 +52,7 @@ rehash(int is_upscaling)
     cur_prime += d;
 
     all_jobs_cap = primes[cur_prime];
-    all_jobs = calloc(all_jobs_cap, sizeof(job));
+    all_jobs = calloc(all_jobs_cap, sizeof(Job *));
     if (!all_jobs) {
         twarnx("Failed to allocate %zu new hash buckets", all_jobs_cap);
         hash_table_was_oom = 1;
@@ -67,7 +67,7 @@ rehash(int is_upscaling)
 
     for (i = 0; i < old_cap; i++) {
         while (old[i]) {
-            job j = old[i];
+            Job *j = old[i];
             old[i] = j->ht_next;
             j->ht_next = NULL;
             store_job(j);
@@ -78,40 +78,48 @@ rehash(int is_upscaling)
     }
 }
 
-job
+Job *
 job_find(uint64 job_id)
 {
-    job jh = NULL;
     int index = _get_job_hash_index(job_id);
+    Job *jh = all_jobs[index];
 
-    for (jh = all_jobs[index]; jh && jh->r.id != job_id; jh = jh->ht_next);
+    while (jh && jh->r.id != job_id)
+        jh = jh->ht_next;
 
     return jh;
 }
 
-job
+Job *
 allocate_job(int body_size)
 {
-    job j;
+    Job *j;
 
-    j = malloc(sizeof(struct job) + body_size);
-    if (!j) return twarnx("OOM"), (job) 0;
+    j = malloc(sizeof(Job) + body_size);
+    if (!j) {
+        twarnx("OOM");
+        return (Job *) 0;
+    }
 
-    memset(j, 0, sizeof(struct job));
+    memset(j, 0, sizeof(Job));
     j->r.created_at = nanoseconds();
     j->r.body_size = body_size;
-    j->next = j->prev = j; /* not in a linked list */
+    j->body = (char *)j + sizeof(Job);
+    job_list_reset(j);
     return j;
 }
 
-job
-make_job_with_id(uint pri, int64 delay, int64 ttr,
-                 int body_size, tube tube, uint64 id)
+Job *
+make_job_with_id(uint32 pri, int64 delay, int64 ttr,
+                 int body_size, Tube *tube, uint64 id)
 {
-    job j;
+    Job *j;
 
     j = allocate_job(body_size);
-    if (!j) return twarnx("OOM"), (job) 0;
+    if (!j) {
+        twarnx("OOM");
+        return (Job *) 0;
+    }
 
     if (id) {
         j->r.id = id;
@@ -131,9 +139,9 @@ make_job_with_id(uint pri, int64 delay, int64 ttr,
 }
 
 static void
-job_hash_free(job j)
+job_hash_free(Job *j)
 {
-    job *slot;
+    Job **slot;
 
     slot = &all_jobs[_get_job_hash_index(j->r.id)];
     while (*slot && *slot != j) slot = &(*slot)->ht_next;
@@ -147,7 +155,7 @@ job_hash_free(job j)
 }
 
 void
-job_free(job j)
+job_free(Job *j)
 {
     if (j) {
         TUBE_ASSIGN(j->tube, NULL);
@@ -158,41 +166,45 @@ job_free(job j)
 }
 
 void
-job_setheappos(void *j, int pos)
+job_setpos(void *j, size_t pos)
 {
-    ((job)j)->heap_index = pos;
+    ((Job *)j)->heap_index = pos;
 }
 
 int
-job_pri_less(void *ax, void *bx)
+job_pri_less(void *ja, void *jb)
 {
-    job a = ax, b = bx;
+    Job *a = (Job *)ja;
+    Job *b = (Job *)jb;
     if (a->r.pri < b->r.pri) return 1;
     if (a->r.pri > b->r.pri) return 0;
     return a->r.id < b->r.id;
 }
 
 int
-job_delay_less(void *ax, void *bx)
+job_delay_less(void *ja, void *jb)
 {
-    job a = ax, b = bx;
+    Job *a = ja;
+    Job *b = jb;
     if (a->r.deadline_at < b->r.deadline_at) return 1;
     if (a->r.deadline_at > b->r.deadline_at) return 0;
     return a->r.id < b->r.id;
 }
 
-job
-job_copy(job j)
+Job *
+job_copy(Job *j)
 {
-    job n;
+    if (!j)
+        return NULL;
 
-    if (!j) return NULL;
+    Job *n = malloc(sizeof(Job) + j->r.body_size);
+    if (!n) {
+        twarnx("OOM");
+        return (Job *) 0;
+    }
 
-    n = malloc(sizeof(struct job) + j->r.body_size);
-    if (!n) return twarnx("OOM"), (job) 0;
-
-    memcpy(n, j, sizeof(struct job) + j->r.body_size);
-    n->next = n->prev = n; /* not in a linked list */
+    memcpy(n, j, sizeof(Job) + j->r.body_size);
+    job_list_reset(n);
 
     n->file = NULL; /* copies do not have refcnt on the wal */
 
@@ -206,7 +218,7 @@ job_copy(job j)
 }
 
 const char *
-job_state(job j)
+job_state(Job *j)
 {
     if (j->r.state == Ready) return "ready";
     if (j->r.state == Reserved) return "reserved";
@@ -215,30 +227,39 @@ job_state(job j)
     return "invalid";
 }
 
-int
-job_list_any_p(job head)
+// job_list_reset detaches head from the list,
+// marking the list starting in head pointing to itself.
+void
+job_list_reset(Job *head)
 {
-    return head->next != head || head->prev != head;
+    head->prev = head;
+    head->next = head;
 }
 
-job
-job_remove(job j)
+int
+job_list_is_empty(Job *head)
+{
+    return head->next == head && head->prev == head;
+}
+
+Job *
+job_list_remove(Job *j)
 {
     if (!j) return NULL;
-    if (!job_list_any_p(j)) return NULL; /* not in a doubly-linked list */
+    if (job_list_is_empty(j)) return NULL; /* not in a doubly-linked list */
 
     j->next->prev = j->prev;
     j->prev->next = j->next;
 
-    j->prev = j->next = j;
+    job_list_reset(j);
 
     return j;
 }
 
 void
-job_insert(job head, job j)
+job_list_insert(Job *head, Job *j)
 {
-    if (job_list_any_p(j)) return; /* already in a linked list */
+    if (!job_list_is_empty(j)) return; /* already in a linked list */
 
     j->prev = head->prev;
     j->next = head;
