@@ -39,7 +39,7 @@ su(const char *user)
 static void
 handle_sigterm_pid1()
 {
-    raise(SIGKILL);
+    exit(143);
 }
 
 static void
