@@ -15,6 +15,7 @@
 #include <inttypes.h>
 #include <stdarg.h>
 #include <signal.h>
+#include <limits.h>
 
 /* job body cannot be greater than this many bytes long */
 size_t job_data_size_limit = JOB_DATA_SIZE_LIMIT_DEFAULT;
@@ -185,13 +186,13 @@ size_t job_data_size_limit = JOB_DATA_SIZE_LIMIT_DEFAULT;
     "binlog-max-size: %d\n" \
     "draining: %s\n" \
     "id: %s\n" \
-    "hostname: %s\n" \
-    "os: %s\n" \
-    "platform: %s\n" \
+    "hostname: \"%s\"\n" \
+    "os: \"%s\"\n" \
+    "platform: \"%s\"\n" \
     "\r\n"
 
 #define STATS_TUBE_FMT "---\n" \
-    "name: %s\n" \
+    "name: \"%s\"\n" \
     "current-jobs-urgent: %" PRIu64 "\n" \
     "current-jobs-ready: %zu\n" \
     "current-jobs-reserved: %" PRIu64 "\n" \
@@ -209,7 +210,7 @@ size_t job_data_size_limit = JOB_DATA_SIZE_LIMIT_DEFAULT;
 
 #define STATS_JOB_FMT "---\n" \
     "id: %" PRIu64 "\n" \
-    "tube: %s\n" \
+    "tube: \"%s\"\n" \
     "state: %s\n" \
     "pri: %u\n" \
     "age: %" PRId64 "\n" \
@@ -278,6 +279,7 @@ static const char * op_names[] = {
     CMD_RESERVE_JOB,
 };
 
+static Job *remove_ready_job(Job *j);
 static Job *remove_buried_job(Job *j);
 
 // epollq_add schedules connection c in the s->conns heap, adds c
@@ -459,13 +461,11 @@ process_queue()
     int64 now = nanoseconds();
 
     while ((j = next_awaited_job(now))) {
-        heapremove(&j->tube->ready, j->heap_index);
-        ready_ct--;
-        if (j->r.pri < URGENT_THRESHOLD) {
-            global_stat.urgent_ct--;
-            j->tube->stat.urgent_ct--;
+        j = remove_ready_job(j);
+        if (j == NULL) {
+            twarnx("job not ready");
+            continue;
         }
-
         Conn *c = ms_take(&j->tube->waiting_conns);
         if (c == NULL) {
             twarnx("waiting_conns is empty");
@@ -1436,11 +1436,12 @@ dispatch_cmd(Conn *c)
 
     case OP_RESERVE_TIMEOUT:
         errno = 0;
-        timeout = strtol(c->cmd + CMD_RESERVE_TIMEOUT_LEN, &end_buf, 10);
-        if (errno) {
+        uint32 utimeout = 0;
+        if (read_u32(&utimeout, c->cmd + CMD_RESERVE_TIMEOUT_LEN, &end_buf) != 0 || utimeout > INT_MAX) {
             reply_msg(c, MSG_BAD_FORMAT);
             return;
         }
+        timeout = (int)utimeout;
         /* Falls through */
 
     case OP_RESERVE:
@@ -1701,7 +1702,7 @@ dispatch_cmd(Conn *c)
         }
         op_ct[type]++;
 
-        t = tube_find(name);
+        t = tube_find(&tubes, name);
         if (!t) {
             reply_msg(c, MSG_NOTFOUND);
             return;
@@ -1795,14 +1796,7 @@ dispatch_cmd(Conn *c)
         }
         op_ct[type]++;
 
-        t = NULL;
-        for (i = 0; i < c->watch.len; i++) {
-            t = c->watch.items[i];
-            if (strncmp(t->name, name, MAX_TUBE_NAME_LEN) == 0)
-                break;
-            t = NULL;
-        }
-
+        t = tube_find(&c->watch, name);
         if (t && c->watch.len < 2) {
             reply_msg(c, MSG_NOT_IGNORED);
             return;
@@ -1831,7 +1825,7 @@ dispatch_cmd(Conn *c)
             reply_msg(c, MSG_BAD_FORMAT);
             return;
         }
-        t = tube_find(name);
+        t = tube_find(&tubes, name);
         if (!t) {
             reply_msg(c, MSG_NOTFOUND);
             return;
